local ATTACK = TUNING.WOODENSWORD_ACTIVATED_ATTACK
local DAPERNESS = TUNING.WOODENSWORD_ACTIVATED_DAPERNESS

local LIGHT =
{
	DEFAULT = nil,
	
	qian = {order=7, r=255, g=255, b=255},
	zhen = {order=5, r=60, g=255, b=255},
	li = {order=1, r=255, g=0, b=0},
	dui = {order=3, r=143, g=25, b=164},
}

local SPEED = 
{
	DEFAULT = TUNING.WOODENSWORD_ACTIVATED_SPEED,
	xun = TUNING.WOODENSWORD_ACTIVATED_XUN_SPEED,
}

local Bagua = Class(function(self, inst)
	self.inst = inst
	self.enabled = {}
	self.effects = {}
	self.extraattack = 0
	self.reactivatelimit = 1
	self.noeffectdays = 0
	self.isnormal = false
	
	self:ResetCurrent()
	
	self.onupdatefn = nil
	self.onturnonlightfn = nil
	self.onturnofflightfn = nil
	
	inst:WatchWorldState("cycles", function() self:OnDay() end)
end)

function Bagua:OnSave()
    return 
	{
		enabled = self.enabled,
        effects = self.effects,
		extraattack = self.extraattack,
		reactivatelimit = self.reactivatelimit,
		noeffectdays = self.noeffectdays,
		isnormal = self.isnormal,
	}
end

function Bagua:OnLoad(data)
	if data then
		self.enabled = data.enabled or {}
		self.effects = data.effects or {}
		self.extraattack = data.extraattack or 0
		self.reactivatelimit = data.reactivatelimit or 1
		self.noeffectdays = data.noeffectdays or 0
		self.isnormal = data.isnormal or false
	end
	self:UpdateEffect()
end

function Bagua:OnDay()
	local hasEffect = false
	for k,v in pairs(self.effects) do
		if v then
			self.effects[k] = self.effects[k] - 1
		end
		if v == 1 then
			self.effects[k] = nil
		else
			hasEffect = true
		end
	end
	if not hasEffect and not self.isnormal then
		self:SetRandomDailyEffect()
	end
	self.reactivatelimit = 1
	self:UpdateEffect()
end

function Bagua:CheckReactivate()
	if self.reactivatelimit <= 0 then
		return false, 0, 0
	end
	local enabledcount = 0
	local effectcount = 0
	for k,v in pairs(self.enabled) do
		enabledcount = enabledcount + 1
		if self.effects[k] then
			effectcount = effectcount + 1
		end
	end
	return effectcount > 0 and effectcount < enabledcount, enabledcount, effectcount
end

local function dump(tb)
--	print("{")
--	for k,v in pairs(tb) do
--		print(tostring(k).."="..tostring(v))
--	end
--	print("}")
end

function Bagua:OnEquip(owner)
	dump(self.enabled)
	dump(self.effects)
	
	if self.onturnonlightfn then
		self.onturnonlightfn(self.inst, self.current_light)
	end
end

function Bagua:OnUnequip(owner)
	if self.onturnofflightfn then
		self.onturnofflightfn(self.inst)
	end
end

function Bagua:OnDrop()
	if self.onturnofflightfn then
		self.onturnofflightfn(self.inst)
	end
end

function Bagua:ResetCurrent()
	self.current_daperness = DAPERNESS.DEFAULT
	self.current_light = LIGHT.DEFAULT
	self.current_status = nil
	self.current_speedmult = SPEED.DEFAULT
	self.current_attack = ATTACK.DEFAULT
end

function Bagua:GetAllEffects()
	return self.effects
end

function Bagua:UpdateEffect()
	local effectcount = 0
	
	self:ResetCurrent()
	
	for k,v in pairs(self.enabled) do
		self.inst:RemoveTag("bagua_" .. k)
	end
	
	for k,v in pairs(self:GetAllEffects()) do
		if DAPERNESS[k] and self.current_daperness < DAPERNESS[k] then
			self.current_daperness = DAPERNESS[k]
		end
		if LIGHT[k] and (not self.current_light or self.current_light.order < LIGHT[k].order) then
			self.current_light = LIGHT[k]
		end
		if ATTACK[k] and self.current_attack < ATTACK[k] then
			self.current_attack = ATTACK[k]
		end
		if SPEED[k] and self.current_speedmult < SPEED[k] then
			self.current_speedmult = SPEED[k]
		end
		effectcount = effectcount + 1
		self.current_status = k
		
		self.inst:AddTag("bagua_" .. k)
	end
	
	if effectcount > 1 then
		self.current_status = "MIXED"
	end
	
	if self:CheckReactivate() and not self.isnormal then
		self.inst:AddTag("reactivatable")
	else
		self.inst:RemoveTag("reactivatable")
	end
	
	self:CallOnUpdate()
end

function Bagua:SetRandomDailyEffect()
	local size = 0
	local effectcount = 1
	for k,v in pairs(self.enabled) do
		size = size + 1
	end
	if size == 0 then
		return
	end
	if size ~= 8 then
		size = size + TUNING.BAGUA_DUMMY_EFFECT[self.noeffectdays + 1]
	else
		effectcount = 2
	end
	local noeffect = true
	while effectcount > 0 do
		local i = size
		for k,v in pairs(self.enabled) do
			if math.random() < 1 / i and not self.effects[k] then
				self.effects[k] = 1
				noeffect = false
				break
			end
			i = i - 1
		end
		effectcount = effectcount - 1
	end
	if noeffect then
		self.noeffectdays = self.noeffectdays + 1
	else
		self.noeffectdays = 0
	end
end

function Bagua:Reactivate(doer)
	local valid, enabledcount, effectcount = self:CheckReactivate()
	if not valid then
		return
	end
	
	local coordcount = enabledcount - effectcount
	local coordeffect = nil
	for k,v in pairs(self.enabled) do
		if not self.effects[k] then
			if math.random() < 1 / coordcount then
				coordeffect = k
				break
			end
			coordcount = coordcount - 1
		end
	end
	
	for k,v in pairs(self.effects) do
		if math.random() < 1 / effectcount then
			self.effects[k] = v - 1
			if v == 1 then
				self.effects[k] = nil
			end
			self.effects[coordeffect] = 1
			break
		end
		effectcount = effectcount - 1
	end
	
	if doer.components.sanity then
		doer.components.sanity:DoDelta(TUNING.REACTIVATE_SANITY_PENALTY)
	end
	
	self.reactivatelimit = self.reactivatelimit - 1
	
	self:UpdateEffect()
end

function Bagua:Copy(doer, target)
	if not target.components.bagua then
		return
	end
	
	local that = target.components.bagua
	that.effects = {}
	that.enabled = {}
	
	for k,v in pairs(self.effects) do
		that:ActivateEffect(k, 3)
	end
	
	if doer.components.sanity then
		doer.components.sanity:DoDelta(TUNING.COPY_SANITY_PENALTY)
	end
end

function Bagua:CallOnUpdate()
	if self.onupdatefn then
		self.onupdatefn(self.inst)
	end
end

function Bagua:ActivateEffect(name, count)
	if not count then
		count = 1
	end
	self.enabled[name] = true
	if self.effects[name] then
		self.effects[name] = self.effects[name] + count
	else
		self.effects[name] = count
	end
	self:UpdateEffect()
end

function Bagua:IncreaseExtraAttack(value)
	self.extraattack = self.extraattack + value
	self:CallOnUpdate()
end

function Bagua:ResetExtraAttack()
	local needCall = (self.extraattack ~= 0)
	self.extraattack = 0
	if needCall then self:CallOnUpdate() end
end

function Bagua:SetOnUpdateFn(func)
	self.onupdatefn = func
end

function Bagua:SetOnTurnOnLight(func)
	self.onturnonlightfn = func
end

function Bagua:SetOnTurnOffLight(func)
	self.onturnofflightfn = func
end

function Bagua:HasEffect(name)
	return self.effects[name]
end

function Bagua:GetDaperness()
	if self.isnormal then
		return self.current_daperness * TUNING.WOODENSWORD_NORMAL_DAPERNESS_MULT
	else
		return self.current_daperness
	end
end

function Bagua:GetStatus()
	return self.current_status
end

function Bagua:GetAttack()
	if self.isnormal then
		return self.current_attack * TUNING.WOODENSWORD_NORMAL_ATTACK_MULT + self.extraattack
	else
		return self.current_attack + self.extraattack
	end
end

function Bagua:GetSpeedMult()
	return self.current_speedmult
end

function Bagua:GetLight()
	return self.current_light
end

return Bagua