local texture = "fx/torchfire.tex"
local shader = "shaders/particle.ksh"
local colour_envelope_name = "liflamecolourenvelope"
local scale_envelope_name = "liflamescaleenvelope"

local assets =
{
	Asset( "IMAGE", texture ),
	Asset( "SHADER", shader ),
}

local max_scale = 5

local function IntColour( r, g, b, a )
	return { r / 255.0, g / 255.0, b / 255.0, a / 255.0 }
end

local init = false
local function InitEnvelope()
	if EnvelopeManager and not init then
		init = true
		EnvelopeManager:AddColourEnvelope(
			colour_envelope_name,
			{	{ 0,	IntColour( 187, 111, 60, 192 ) },
				{ 0.49,	IntColour( 187, 111, 60, 192 ) },
				{ 0.5,	IntColour( 255, 255, 0, 192 ) },
				{ 0.51,	IntColour( 255, 255, 56, 192 ) },
				{ 0.75,	IntColour( 255, 192, 56, 128 ) },
				{ 1,	IntColour( 255, 128, 28, 0 ) },
			} )

		EnvelopeManager:AddVector2Envelope(
			scale_envelope_name,
			{
				{ 0,	{ max_scale * 0.5, max_scale } },
				{ 1,	{ max_scale * 0.5 * 0.5, max_scale * 0.5 } },
			} )
	end
end

local max_lifetime = 0.6
--local ground_height = 0.1

local function fn(Sim)
	local inst = CreateEntity()
	inst:AddTag("FX")
	local trans = inst.entity:AddTransform()
    inst.entity:AddNetwork()
	local emitter = inst.entity:AddParticleEmitter()

	InitEnvelope()

	emitter:SetRenderResources( texture, shader )
	emitter:SetMaxNumParticles( 64 )
	emitter:SetMaxLifetime( max_lifetime )
	emitter:SetColourEnvelope( colour_envelope_name )
	emitter:SetScaleEnvelope( scale_envelope_name );
	emitter:SetBlendMode( BLENDMODE.Additive )
	emitter:EnableBloomPass( true )
	emitter:SetUVFrameSize( 1.0 / 4.0, 1.0 )

	-----------------------------------------------------
	local tick_time = TheSim:GetTickTime()

	local desired_particles_per_second = 64
	local particles_per_tick = desired_particles_per_second * tick_time

	local emitter = inst.ParticleEmitter

	local num_particles_to_emit = 1
	
	local sphere_emitter = CreateSphereEmitter( 1 )

	local emit_fn = function()
		local vx, vy, vz = 0.01 * UnitRand(), 0.015 + 0.005 * UnitRand(), 0.01 * UnitRand()
		local lifetime = max_lifetime * ( 0.9 + UnitRand() * 0.1 )
		local px, py, pz

		px, py, pz = sphere_emitter()
		py = py * 0.4 + 0.4

		local u_offset = math.random( 0, 3 ) * 0.25
		local v_offset = 0

		emitter:AddParticleUV(
			lifetime,			-- lifetime
			px, py, pz,			-- position
			vx, vy, vz,			-- velocity
			u_offset, v_offset	-- uv offset
		)
	end
	
	local updateFunc = function()
		if inst._life:value() < max_lifetime then
			return
		end
		
		while num_particles_to_emit > 1 do
			emit_fn( emitter )
			num_particles_to_emit = num_particles_to_emit - 1
		end

		num_particles_to_emit = num_particles_to_emit + particles_per_tick
	end

	EmitterManager:AddEmitter( inst, nil, updateFunc )
	
	local light = inst.entity:AddLight()
    light:SetIntensity(.75)
    light:SetFalloff(0.5)
    light:SetRadius(3)
	light:SetColour(255/255, 192/255, 0/255)
	
	inst._life = net_float(inst.GUID, "limitedlife.life", "onlimitedlifedirty")

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end
	
	inst:AddComponent("liflame")
	inst.components.liflame.ontargetdead = function(inst)
		inst.components.limitedlife.life = max_lifetime
	end

	inst:AddComponent("limitedlife")
	inst.components.limitedlife.life = TUNING.LIFLAME_LIFETIME
    
    return inst
end

return Prefab( "common/fx/liflame", fn, assets) 
 
