
local MakePlayerCharacter = require "prefabs/player_common"


local assets = {
	Asset("ANIM", "anim/wenyu.zip"),
	Asset("ANIM", "anim/ghost_wenyu_build.zip"),
}

local prefabs = {
	"woodensword",
}

local start_inv = {
	"woodensword",
}

local function arr2map(data)
	local result = {}
	for k,v in pairs(data) do
		result[v] = true
	end
	return result
end

local inncents = arr2map({
	-- DS
	"bee", "butterfly", "beefalo", "babybeefalo",
	"crow", "robin", "robin_winter",
	"frog", "fireflies", "perd", "koalefant_summer", "koalefant_winter", "penguin",
	"rabbit", "monkey",
	-- RoG
	"catcoon", "mole", "lightninggoat", "buzzard",
	-- SW
	"parrot", "parrot_pirate", "seagull", "toucan",
	"whale_blue", "ballphin", "primeape", "ox", "crab",
	"doydoy", "doydoybaby", "jellyfish", "lobster"
})

local function isInnocent(inst, victim)
	if victim.prefab == "rabbit" then
		return not (inst.components.sanity and inst.components.sanity:IsCrazy())
	elseif victim.prefab == "monkey" then
		return not victim:HasTag("nightmare")
	elseif victim.prefab == "lightninggoat" then
		return not victim:HasTag("charged")
	elseif victim.prefab == "crab" then
		return victim.iscrab
	else
		return inncents[victim.prefab]
	end
end

local forbiddenfoods = arr2map({
	-- old and giant fish
	"fish", "eel", "fish_cooked", "eel_cooked",
	-- sw fish
	"tropical_fish",
	"fish_raw", "fish_med", "fish_med_cooked", "swordfish", "shark_fin",
	"fish_raw_small",
	-- old and giant cooked fish
	"fishsticks", "fishtacos", "unagi",
	-- sw cooked fish
	"californiaroll", "seafoodgumbo", "ceviche", "surfnturf",
})

function WenyuCanEatFn(inst, food)
	return not forbiddenfoods[food.prefab]
end

local function common_postinit(inst)
	-- choose which sounds this character will play
	inst.soundsname = "wilson"

	-- Minimap icon
	inst.MiniMapEntity:SetIcon( "wenyu.tex" )
	
	-- Recipes Enabled
	inst:AddTag("taoist")
end

local function master_postinit(inst)
	-- Stats	
	inst.components.health:SetMaxHealth(TUNING.WENYU_MAX_HEALTH)
	inst.components.hunger:SetMax(TUNING.WENYU_MAX_HUNGER)
	inst.components.sanity:SetMax(TUNING.WENYU_MAX_SANITY)
	
	-- Is less afraid of monsters
	inst.components.sanity.neg_aura_mult = TUNING.WENYU_NEG_AURA_MULT
	
	-- Start items
	-- Since the item can be used, no need to guarantee it.
	-- inst.components.inventory:GuaranteeItems(start_inv)
	
	-- Is less afraid of monster meats
	inst:ListenForEvent("oneat", function(inst, data)
		local food = data.food
		if (food.prefab == "monstermeat" or food.prefab == "cookedmonstermeat" or food.prefab == "monstermeat_dried") and
				inst.components.sanity.current > 0 then
			local sanitydelta = food.components.edible:GetSanity(inst)
			inst.components.sanity:DoDelta(-1 * sanitydelta)
			inst.components.sanity:DoDelta(TUNING.WENYU_FOOD_NEG_SANITY_MULT * sanitydelta)
		end
	end)
	
	-- Doesn't eat fish
	inst.components.eater:SetDiet({ FOODTYPE.TAOIST }, { FOODTYPE.TAOIST })
	
	-- Doesn't kill animals
	inst:ListenForEvent("killed", function(inst, data)
		local victim = data.victim
		if isInnocent(inst, victim) then
			inst.components.talker:Say(GetString(inst.prefab, "ANNOUNCE_TAOISTICDISCIPLINES", "KILL"))
			inst.components.sanity:DoDelta(TUNING.WENYU_KILL_ANIMAL_PENALTY)
		end
	end)
	inst:ListenForEvent("onattackother", function(inst, data)
		local target = data.target
		if isInnocent(inst, target) then
			inst.components.talker:Say(GetString(inst.prefab, "ANNOUNCE_TAOISTICDISCIPLINES", "ATTACK"))
			inst.components.sanity:DoDelta(TUNING.WENYU_HURT_ANIMAL_PENALTY)
		end
	end)
	
	-- Recipes
	local savedbagua = nil
	inst:ListenForEvent("makerecipe", function(inst, data)
		local recipe = data.recipe
		if recipe.name:sub(1, 22) ~= "woodensword_activated_" then
			return
		end
		local inventory = inst.components.inventory
		local v = inventory:FindItem(function(v)
			return v and v.prefab == "woodensword_activated"
		end)
		if v then
			savedbagua = v.components.bagua:OnSave()
		end
	end)
	inst:ListenForEvent("builditem", function(inst, data)
		local item = data.item
		local recipe = data.recipe
		if recipe.name:sub(1, 22) ~= "woodensword_activated_" then
			return
		end
		if savedbagua then item.components.bagua:OnLoad(savedbagua) end
		savedbagua = nil
		item.components.bagua:ActivateEffect(recipe.name:sub(23))
	end)
end

return MakePlayerCharacter("wenyu", prefabs, assets, common_postinit, master_postinit, start_inv)
